@@ -1,7 +1,7 @@
 from __future__ import absolute_import, division, print_function, unicode_literals
 
 import os.path
-from sys import version_info
+from sys import version_info, stdout
 from os import fdopen
 from io import open, TextIOWrapper, TextIOBase, RawIOBase, BufferedIOBase, SEEK_SET, SEEK_END
 from typing import TYPE_CHECKING
@@ -78,13 +78,6 @@ def read_or_raise(fin, size):
 
 	return data
 
-def linefileiter(fin, check_last=True): # kinda unnecessary
-	last = None
-	for line in fin:
-		yield line
-		last = line
-	assert last.endswith("\n"), "File does not end in newline: {}".format(last)
-
 def get_file_range(path, start, size):
 	# type: (PathType, int, int) -> bytes
 
@@ -100,7 +93,9 @@ def truncate_file(path, size):
 
 def wrap_text(bf, mode, encoding, errors, newline):
 	if "t" in mode:
-		return TextIOWrapper(bf, encoding, errors, newline)
+		tf = TextIOWrapper(bf, encoding, errors, newline)
+		tf.mode = mode
+		return tf
 	return bf
 
 def copen(file, mode="rt", archive_file=None, encoding=None, errors=None, newline=None, compresslevel=9, ext=None):
@@ -118,7 +113,7 @@ def copen(file, mode="rt", archive_file=None, encoding=None, errors=None, newlin
 			to enable transparent compression
 
 		Returns a file-like object. If `file` was a fd,
-			it will be closed if the file-like object is closed.
+			it will be closed when the file-like object is closed.
 	"""
 
 	encoding = _check_arguments(mode, encoding)
@@ -155,6 +150,9 @@ def copen(file, mode="rt", archive_file=None, encoding=None, errors=None, newlin
 
 		return wrap_text(bf, mode, encoding, errors, newline)
 
+	if isinstance(file, TextIOWrapper):
+		return file
+
 	if hasattr(file, "read") or hasattr(file, "write"): # file should be in binary mode here
 		return wrap_text(file, mode, encoding, errors, newline)
 
@@ -197,23 +195,23 @@ class OpenFileAndDeleteOnError(object):
 
 class OptionalWriteOnlyFile(object):
 
-	def __init__(self, path=None, mode="xb", encoding=None, compresslevel=9):
-		# type: (Optional[PathType], str, Optional[str], int) -> None
+	def __init__(self, path=None, mode="xb", encoding=None, errors=None, newline=None, compresslevel=9):
+		# type: (Optional[PathType], str, Optional[str], Optional[str], Optional[str], int) -> None
 
 		if path:
-			self.f = copen(path, mode, encoding=encoding, compresslevel=compresslevel)
+			self.fp = copen(path, mode, encoding=encoding, errors=errors, newline=newline, compresslevel=compresslevel)
 		else:
-			self.f = None
+			self.fp = None
 
 	def __enter__(self):
-		if self.f:
-			return self.f
+		if self.fp:
+			return self.fp
 		else:
 			return self
 
 	def __exit__(self, exc_type, exc_value, traceback):
-		if self.f:
-			self.f.close()
+		if self.fp:
+			self.fp.close()
 
 	def write(self, data):
 		pass
@@ -221,23 +219,47 @@ class OptionalWriteOnlyFile(object):
 	def seek(self, offset, whence=None):
 		pass
 
+class StdoutFile(object):
+
+	def __init__(self, path=None, mode="xb", encoding=None, errors=None, newline=None, compresslevel=9):
+		# type: (Optional[PathType], str, Optional[str], Optional[str], Optional[str], int) -> None
+
+		if path:
+			self.fp = copen(path, mode, encoding=encoding, errors=errors, newline=newline, compresslevel=compresslevel)
+			self.doclose = True
+		else:
+			self.doclose = False
+			if "b" in mode:
+				self.fp = stdout.buffer
+			elif "t" in mode:
+				self.fp = stdout
+			else:
+				raise ValueError("Explicit text or binary mode required: {}".format(mode))
+
+	def __enter__(self):
+		return self.fp
+
+	def __exit__(self, exc_type, exc_value, traceback):
+		if self.doclose:
+			self.fp.close()
+
 class PathOrBinaryIO(object):
 
 	def __init__(self, fname, mode="rb", close=False):
 		# type: (Union[PathType, BinaryIO],  str, str, str, Optional[str], bool) -> None
 
 		if isinstance(fname, (RawIOBase, BufferedIOBase)):
-			self._close = close
+			self.doclose = close
 			self.fp = fname
 		else:
-			self._close = True
+			self.doclose = True
 			self.fp = copen(fname, mode)
 
 	def __enter__(self):
 		return self.fp
 
 	def __exit__(self, exc_type, exc_value, traceback):
-		if self._close:
+		if self.doclose:
 			self.fp.close()
 
 class PathOrTextIO(object):
@@ -246,17 +268,17 @@ class PathOrTextIO(object):
 		# type: (Union[PathType, TextIO], str, str, str, Optional[str], bool) -> None
 
 		if isinstance(fname, TextIOBase):
-			self.close = close
+			self.doclose = close
 			self.fp = fname
 		else:
-			self.close = True
+			self.doclose = True
 			self.fp = copen(fname, mode, encoding=encoding, errors=errors, newline=newline)
 
 	def __enter__(self):
 		return self.fp
 
 	def __exit__(self, exc_type, exc_value, traceback):
-		if self.close:
+		if self.doclose:
 			self.fp.close()
 
 class LastLineFile(object):
@@ -664,7 +686,7 @@ def iter_zip(file, mode="rb", encoding=None, errors=None, newline=None, password
 
 	with ZipFile(file, newmode) as zf:
 		for zi in zf.infolist():
-			if not zi.is_dir():  # fixeme: Py3.5 AttributeError: 'ZipInfo' object has no attribute 'is_dir'
+			if not zi.is_dir():  # fixme: Py3.5 AttributeError: 'ZipInfo' object has no attribute 'is_dir'
 				with zf.open(zi, newmode, password) as bf:
 					yield wrap_text(bf, mode, encoding, errors, newline)
 
@@ -695,6 +717,48 @@ def iter_tar(file, mode="rb", encoding=None, errors=None, newline=None):
 				with tf.extractfile(ti) as bf: # no `mode` for `extractfile`
 					yield wrap_text(bf, mode, encoding, errors, newline)
 
+def iter_lines(path, encoding="utf-8", errors="strict", newline=None, verbose=False):
+	# type: (str, str, str, Optional[str], bool) -> Iterator[str]
+
+	""" Yield lines from text file. Handles compressed files as well.
+		if verbose is True, print a progress bar
+	"""
+
+	with copen(path, "rt", encoding=encoding, errors=errors, newline=newline) as fr:
+
+		if verbose:
+			from tqdm import tqdm
+
+			prev_pos = 0
+
+			fr.seek(0, os.SEEK_END)
+			total = fr.tell()
+			fr.seek(0, os.SEEK_SET)
+
+			with tqdm(total=total, leave=False, unit="B", unit_scale=True) as pbar:
+				while True:  # cannot use `for line in fr` here
+					line = fr.readline()
+					if not line:
+						break
+					cur_pos = fr.tell()
+					pbar.update(cur_pos - prev_pos)
+					yield line
+					prev_pos = cur_pos
+		else:
+			for line in fr:
+				yield line
+
+def iter_stripped(path, encoding="utf-8", errors="strict", newline=None):
+	# type: (str, str, str, Optional[str]) -> Iterator[str]
+
+	""" Yield lines from text file with trailing newlines stripped.
+		Handles compressed files as well.
+	"""
+
+	with copen(path, "rt", encoding=encoding, errors=errors, newline=newline) as fr:
+		for line in fr:
+			yield line.rstrip("\r\n")
+
 # is this still needed?
 def file_byte_reader(filename, inputblocksize, outputblocksize, DEBUG=True):
 	# type: (PathType, int, int, bool) -> Iterator[bytes]
