@@ -14,6 +14,7 @@ from types import GeneratorType
 from typing import TYPE_CHECKING
 
 from .exceptions import IteratorExhausted, EmptyIterable
+from .ops import operator_in
 
 if TYPE_CHECKING:
 	from typing import (Any, Callable, Iterable, Iterator, TypeVar, Tuple, Sequence,
@@ -27,7 +28,6 @@ if TYPE_CHECKING:
 logger = logging.getLogger(__name__)
 logger.addHandler(logging.NullHandler())
 
-# was: iterrandint
 def iterrandrange(a, b):
 	# type: (int, int) -> Iterator[int]
 
@@ -184,21 +184,26 @@ def asc_peaks(iterable):
 		cur = x
 	yield cur
 
-def extrema(iterable, first=None, last=None, derivatives={1, -1}):
+def extrema(iterable, first={1, -1}, last={1, -1}, derivatives={1, -1}):
+	# type: (Iterable[T], Set[int], Set[int], Set[int]) -> Iterator[T]
+
 	it = iter(iterable)
 	try:
 		old = next(it)
 	except StopIteration:
 		return
 	try:
-		old_d = diffsgn(old, next(it))
+		old_2 = next(it)
+		old_d = diffsgn(old, old_2)
 	except StopIteration:
 		yield old # fixme: should this be returned?
 		return
 
-	if old_d == first:
+	if old_d in first:
 		yield old
 
+	old = old_2
+
 	for new in it:
 		new_d = diffsgn(old, new)
 		if new_d != old_d and old_d in derivatives:
@@ -206,7 +211,7 @@ def extrema(iterable, first=None, last=None, derivatives={1, -1}):
 		old = new
 		old_d = new_d
 
-	if old_d == last:
+	if old_d in last:
 		yield old
 
 def peaks(it):
@@ -214,14 +219,14 @@ def peaks(it):
 
 	""" Yields peaks of `it`. """
 
-	return extrema(it, -1, 1, {1})
+	return extrema(it, {-1}, {1}, {1})
 
 def valleys(it):
 	# type: (Iterable[T], ) -> Iterator[T]
 
 	""" Yields valleys of `it`. """
 
-	return extrema(it, 1, -1, {-1})
+	return extrema(it, {1}, {-1}, {-1})
 
 def empty():
 	# type: () -> Iterator
@@ -246,7 +251,6 @@ def lastdefault(it, default=None):
 		ret = i
 	return ret
 
-# was: iterlast
 def last(it):
 	# type: (Iterable[T], ) -> T
 
@@ -259,11 +263,11 @@ def last(it):
 	except IndexError:
 		raise_from(EmptyIterable("Empty iterable"), None)
 
-def batch(it, n, filter=None):
+def batch(it, n, func=None):
 	# type: (Iterable[Any], int, Optional[Callable]) -> Iterator[Any] # return type cannot be more specific because of filter()
 
 	""" Batches iterable `it` into batches of size `n`.
-		Optionally post-processes batch with `filter`.
+		Optionally post-processes batch with `func`.
 	"""
 
 	it = iter(it)
@@ -273,10 +277,13 @@ def batch(it, n, filter=None):
 			first_el = next(chunk_it)
 		except StopIteration:
 			return
-		if filter:
-			yield filter(chain((first_el,), chunk_it))
-		else:
-			yield chain((first_el,), chunk_it)
+
+		batch_iter = chain((first_el,), chunk_it)
+
+		if func:
+			batch_iter = func(batch_iter)
+
+		yield batch_iter
 
 def advance(it, n):
 	# type: (Iterable[T], int) -> None
@@ -340,7 +347,6 @@ def findfirst(func, it, default=(None, None)):
 
 	return default
 
-#was: isempty
 def is_empty(it):
 	# type: (Iterator, ) -> bool
 
@@ -507,7 +513,6 @@ def decompress(selectors, data, default=None):
 	except StopIteration: # exception converted because StopIterations raised in generators will cause RuntimeErrors
 		raise IteratorExhausted
 
-#was: get_first_non_none
 def first_not_none(it, default=None):
 	# type: (Iterable[T], Optional[T]) -> Optional[T]
 
@@ -545,14 +550,6 @@ def powerset(sit):
 
 	return chain.from_iterable(combinations(sit, r) for r in range(len(sit)+1))
 
-def any_in_common(a, b):
-	# type: (Iterable[T], Iterable[T]) -> bool
-
-	""" Tests if iterables `a` and `b` have any elements in common.
-	"""
-
-	return any(i == j for i, j in product(a, b))
-
 # had cmp=b"\0" default before
 def all_equal_to(it, cmp):
 	# type: (Iterable[T], T) -> bool
@@ -562,7 +559,6 @@ def all_equal_to(it, cmp):
 
 	return all(elm == cmp for elm in it)
 
-# was: iter_same
 def iter_equal(*its):
 	# type: (*Iterable, ) -> bool
 
@@ -600,14 +596,18 @@ def split(it, size):
 	copies = tee(it, size)
 	return tuple(every_n(it, size, pos) for it, pos in zip(copies, range(size)))
 
-def remove_all_dupes(it): # was: remove_dup_list
-	# type: (Iterable[T], ) -> Iterable[T]
+def no_dupes(*its):
+	# type: (*Iterable[T], ) -> Iterator[T]
 
-	""" Removes all duplicates from `it` while preserving order. """
+	""" Merge `its` and removes all duplicates while preserving order. """
 
-	# Dave Kirby
-	seen = set()
-	return (x for x in it if x not in seen and not seen.add(x))
+	seen = set()  # type: Set[T]
+
+	for it in its:
+		for x in it:
+			if x not in seen:
+				yield x
+				seen.add(x)
 
 def retrier(waittime, attempts=-1, multiplier=1, jitter=0, max_wait=None, jitter_dist="uniform", waitfunc=sleep):
 	# type: (float, int, float, float, Optional[float], str, Callable[[float], Any]) -> Iterator[None]
@@ -646,3 +646,110 @@ def retrier(waittime, attempts=-1, multiplier=1, jitter=0, max_wait=None, jitter
 
 		waittime *= multiplier
 		jitter *= multiplier
+
+def collapse_any(it, col_set):
+	# type: (Iterable[T], Set[T]) -> Iterator[T]
+
+	""" Removes consecutive elements from iterable `it` if element is in `col_set`.
+		([1,1,2,2,3,3,4,4], {1,2}) -> 1, 2, 3, 3, 4, 4
+	"""
+
+	assert isinstance(col_set, set)
+
+	for key, g in groupby(it):
+		if key in col_set:
+			yield key
+		else:
+			for i in g: # yield from g
+				yield i
+
+def collapse_all(it, col_set, replacement):
+	# type: (Iterable[T], Set[T], T) -> Iterator[T]
+
+	""" Removes consecutive elements from iterable `it` if element is in `col_set`
+		and replace `replacement`.
+		([1,1,2,2,3,3,4,4], {1,2}, 5) -> 5, 3, 3, 4, 4
+	"""
+
+	assert isinstance(col_set, set)
+
+	for key, g in groupby(it, key=operator_in(col_set)):
+		if key:
+			yield replacement
+		else:
+			for i in g: # yield from g
+				yield i
+
+def any_in_common(a, b):
+	# type: (Iterable[T], Iterable[T]) -> bool
+
+	""" Tests if iterables `a` and `b` have any elements in common.
+		See also: `any_in`.
+	"""
+
+	return any(i == j for i, j in product(a, b))
+
+def any_in(it, container):
+	# type: (Iterable[T], Container[T]) -> bool
+
+	""" Tests if any elements of `it` are in `container`.
+		For best performance, `container` should be a set.
+		See also: `any_in_common`.
+	"""
+
+	if isinstance(it, set) and isinstance(container, set):
+		return not it.isdisjoint(set)
+
+	return any(elm in container for elm in it)
+
+class CountIter(object):
+
+	""" Example:
+		c = CountIter()
+		with open("asd.txt", "rt", encoding="utf-8") as fr:
+			for line in c.count(fr):
+				func(line)  # do something with line
+		print("Processed", c.get(), "lines")
+	"""
+
+	def __init__(self):
+		self.i = 0
+
+	def count(self, it):
+		for elm in it:
+			self.i += 1
+			yield elm
+
+	def get(self):
+		return self.i
+
+def select_by_indices(it, indices):
+	# type: (Iterable[T], Iterator[int]) -> Iterator[T]
+
+	try:
+		sel_ind = next(indices)
+		for i, elem in enumerate(it):
+			if i == sel_ind:
+				yield elem
+				sel_ind = next(indices)
+	except StopIteration:
+		pass
+
+def skip_by_indices(it, indices):
+	# type: (Iterator[T], Iterator[int]) -> Iterator[int, T]
+
+	i = 0
+	try:
+		sel_ind = next(indices)
+		for elem in it:
+			if i == sel_ind:
+				sel_ind = next(indices)
+				continue
+
+			yield i, elem
+			i += 1
+	except StopIteration:
+		pass
+
+	for ii, elem in enumerate(it, i):
+		yield ii, elem
