@@ -77,7 +77,10 @@ def _islink(path):
 def _uncabspath(path):
 	# type: (str, )-> str
 
-	return unc_prefix + os.path.abspath(path)
+	if path.startswith(unc_prefix):
+		return os.path.abspath(path)
+	else:
+		return unc_prefix + os.path.abspath(path)
 
 def _lock(fp, exclusive=True, block=False):
 	# type: (IO, bool, bool) -> None
@@ -145,7 +148,8 @@ def _disk_usage_windows(path):
 def _volume_info_windows(path):
 	# type: (str, ) -> _volumeinfotuple
 
-	assert path.endswith("\\"), "X: usually doesn't work. X:\\ does."
+	if not path.endswith("\\"):
+		raise ValueError("X: usually doesn't work. X:\\ does.")
 
 	RootPathName = LPCWSTR(path)
 	VolumeNameBuffer = create_unicode_buffer(MAX_PATH+1)
