@@ -2,13 +2,28 @@ from __future__ import absolute_import, division, print_function, unicode_litera
 
 from typing import TYPE_CHECKING
 
+from pandas import DataFrame, Series
+
 from .func import identity
 
 if TYPE_CHECKING:
-	from typing import Callable, Optional, List, TypeVar
-	from pandas import DataFrame
+	from typing import Any, Callable, Iterable, List, Optional, Tuple, TypeVar
 	T = TypeVar("T")
 
+def pandas_json(obj):
+	# type: (Any, ) -> Any
+
+	""" Can be used for the `json.dump` `default` argument
+		to make some pandas objects JSON serializable.
+	"""
+
+	if isinstance(obj, DataFrame):
+		return dict(obj)
+	elif isinstance(obj, Series):
+		return tuple(obj)
+
+	raise TypeError("object of type {} cannot be JSON serialized: {}".format(type(obj), obj))
+
 def strlist(sep):
 	# type: (str, ) -> Callable[[str], List[str]]
 
