@@ -1,6 +1,6 @@
 from __future__ import absolute_import, division, print_function, unicode_literals
 
-from builtins import range, input
+from builtins import range
 from future.moves.itertools import zip_longest
 from future.utils import PY2
 
@@ -16,7 +16,6 @@ from time import sleep
 from io import TextIOWrapper
 from typing import TYPE_CHECKING
 
-from .ops import logical_xor, logical_implication
 from .file import equal_files, _check_arguments
 from .signal import HandleKeyboardInterrupt # problem here
 
@@ -86,6 +85,8 @@ class MyTestRunner(TextTestRunner):
 class NoRaise(object):
 
 	def __init__(self, testcase, message=None):
+		# type: (TestCase, Optional[str]) -> None
+
 		self.testcase = testcase
 		self.message = message
 
@@ -99,6 +100,30 @@ class NoRaise(object):
 			else:
 				self.testcase.fail(exc_value)
 
+def make_comparable(d):
+	# type: (Any, ) -> Any
+
+	if isinstance(d, (str, int, type(None))):
+		return d
+	elif isinstance(d, list):
+		return list(make_comparable(i) for i in d)
+	elif isinstance(d, tuple):
+		return tuple(make_comparable(i) for i in d)
+	elif isinstance(d, dict):
+		return make_comparable(list(d.items()))
+	else:
+		raise ValueError("must be list, tuple or dict, not {}".format(type(d)))
+
+def is_equal_unordered(seq_a, seq_b):
+	# type: (Collection, Collection) -> bool
+
+	if isinstance(seq_a, set) and isinstance(seq_b, set):
+		return seq_a == seq_b
+
+	seq_a = sorted(make_comparable(seq_a))
+	seq_b = sorted(make_comparable(seq_b))
+	return seq_a == seq_b
+
 class MyTestCase(TestCase):
 
 	def subTest(self, msg=None, **params):
@@ -114,8 +139,15 @@ class MyTestCase(TestCase):
 
 		self.assertTrue(first and second, msg)
 
-	def assertFilesEqual(self, first_path, second_path):
-		self.assertTrue(equal_files(first_path, second_path))
+	def assertTypeEqual(self, first, second, msg=None):
+		# type: (Any, Any, Optional[str]) -> None
+
+		self.assertEqual(type(first), type(second), msg)
+
+	def assertFilesEqual(self, first_path, second_path, msg=None):
+		# type: (Any, Any, Optional[str]) -> None
+
+		self.assertTrue(equal_files(first_path, second_path), msg)
 
 	def assertIterEqual(self, first, second, msg=None):
 		# type: (Iterable, Iterable, Optional[str]) -> None
@@ -181,13 +213,26 @@ class MyTestCase(TestCase):
 	def assertUnorderedSeqEqual(self, first, second, msg=None):
 		# type: (Iterable, Iterable, Optional[str]) -> None
 
+		# fixme: use is_equal_unordered
+
 		first = sorted(first)
 		second = sorted(second)
 		self.assertEqual(first, second, msg)
 
 	def assertNoRaise(self, msg=None):
+		# type: (Optional[str], ) -> NoRaise
+
 		return NoRaise(self, msg)
 
+	def assertEqualsOneOf(self, result, truths):
+		# type: (T, Iterable[T]) -> None
+
+		for truth in truths:
+			if result == truth:
+				return
+
+		raise AssertionError("Result does not match one of the provided truths") # from None
+
 def random_arguments(n, *funcs):
 	# type: (int, *Callable[[], Any]) -> Callable[[Callable], Callable]
 
