@@ -16,6 +16,24 @@ if TYPE_CHECKING:
 
 	class Comparable(Protocol):
 
+		def __eq__(self, other):
+			# type: (Any, ) -> bool
+			...
+
+		def __ne__(self, other):
+			# type: (Any, ) -> bool
+			...
+
+	class Orderable(Protocol):
+
+		def __lt__(self, other):
+			# type: (Any, ) -> bool
+			...
+
+		def __gt__(self, other):
+			# type: (Any, ) -> bool
+			...
+
 		def __le__(self, other):
 			# type: (Any, ) -> bool
 			...
